<?php

namespace Litecms\Productsearch\Providers;

use Illuminate\Support\ServiceProvider;

class ProductsearchServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'productsearch');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'productsearch');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerProductsearch();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerProductsearch()
    {
        $this->app->bind('productsearch', function($app) {
            return new Productsearch($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Productsearch', 'Lavalite\Productsearch\Facades\Productsearch');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('litecms.productsearch', function ($app) {
            return $this->app->make('Litecms\Productsearch\Productsearch');
        });

                // Bind ProductCustomSearchresult to repository
        $this->app->bind(
            'Litecms\Productsearch\Interfaces\ProductCustomSearchresultRepositoryInterface',
            \Litecms\Productsearch\Repositories\Eloquent\ProductCustomSearchresultRepository::class
        );

        $this->app->register(\Litecms\Productsearch\Providers\AuthServiceProvider::class);
                $this->app->register(\Litecms\Productsearch\Providers\EventServiceProvider::class);
        
        $this->app->register(\Litecms\Productsearch\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and productsearch's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'litecms.productsearch'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeProductsearch::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['litecms.productsearch'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('litecms/productsearch.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/productsearch')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/productsearch')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
